package explorviz.live_trace_processing.connector;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.MalformedURLException;
import java.net.URL;
import java.nio.ByteBuffer;
import java.nio.channels.SocketChannel;

import explorviz.live_trace_processing.Constants;
import explorviz.live_trace_processing.configuration.Configuration;
import explorviz.live_trace_processing.filter.AbstractSink;
import explorviz.live_trace_processing.record.IRecord;
import explorviz.live_trace_processing.record.ISerilizableRecord;
import explorviz.live_trace_processing.writer.IWriter;

public class TCPConnector extends AbstractSink implements IWriter {
	private URL providerURL;

	private SocketChannel socketChannel;

	private final ByteBuffer buffer = ByteBuffer
			.allocateDirect(Constants.MONITORING_MESSAGE_BUFFER_SIZE);

	private volatile boolean shouldDisconnect = false;

	public TCPConnector(final String hostname, final int port, final Configuration configuration) {
		try {
			setProviderURL(new URL("http://" + hostname + ":" + port));
		} catch (final MalformedURLException e) {
			e.printStackTrace();
		}
	}

	@Override
	public URL getProviderURL() {
		return providerURL;
	}

	@Override
	public void setProviderURL(final URL providerURL) {
		this.providerURL = providerURL;
	}

	@Override
	public void connect() throws IOException {
		while (shouldDisconnect) {
			try {
				Thread.sleep(1);
			} catch (final InterruptedException e) {
			}
		}

		socketChannel = SocketChannel.open(new InetSocketAddress(getProviderURL().getHost(),
				getProviderURL().getPort()));
		// StringRegistry.sendOutAllStringRegistryRecords(); TODO
	}

	@Override
	protected void processRecord(final IRecord record) {
		if (record instanceof ISerilizableRecord) {
			final ISerilizableRecord serilizableRecord = (ISerilizableRecord) record;
			if (buffer.remaining() < serilizableRecord.getRecordSizeInBytes()) {
				buffer.flip();
				send(buffer);
				buffer.clear();
			}
			serilizableRecord.putIntoByteBuffer(buffer, null); // TODO
		}
	}

	private void send(final ByteBuffer buffer) {
		while ((socketChannel == null) || (!socketChannel.isConnected())) {
			try {
				Thread.sleep(1);
			} catch (final InterruptedException e) {
			}
		}

		try {
			while (buffer.hasRemaining()) {
				socketChannel.write(buffer);
			}
			doDisconnectIfNessecary();
		} catch (final IOException e) {
			System.out.println("WARNING: Connection was closed - possible data loss");
			// TODO reconnect in non-load-balancing mode
			try {
				socketChannel.close();
			} catch (final IOException e1) {
			}
		}
	}

	private void doDisconnectIfNessecary() {
		if (shouldDisconnect) {
			if ((socketChannel != null) && socketChannel.isConnected()) {
				try {
					socketChannel.close();
				} catch (final IOException e) {
					e.printStackTrace();
				}
			}
			shouldDisconnect = false;
		}
	}

	@Override
	public final void disconnect() {
		if (socketChannel != null) {
			shouldDisconnect = true;
		}
	}
}
