package explorviz.live_trace_processing.main;

import java.io.IOException;

import explorviz.live_trace_processing.configuration.Configuration;
import explorviz.live_trace_processing.configuration.ConfigurationFactory;
import explorviz.live_trace_processing.connector.TCPConnector;
import explorviz.live_trace_processing.reader.TCPReader;
import explorviz.live_trace_processing.writer.load_balancer.LoadBalancer;

public class WorkerStarter {

	public static void main(final String[] args) {
		final Configuration configuration = ConfigurationFactory.createSingletonConfiguration();

		final boolean worker = configuration
				.getBooleanProperty(ConfigurationFactory.WORKER_ENABLED);

		if (worker) {
			final TCPConnector tcpConnector = new TCPConnector(
					configuration.getStringProperty(ConfigurationFactory.WRITER_TARGET_IP),
					configuration.getIntProperty(ConfigurationFactory.WRITER_TARGET_PORT),
					configuration);

			configureLoadBalancerIfEnabled(configuration, tcpConnector);

			new TCPReader(10133, tcpConnector).read();
		} else { // testing purpose
			new TCPReader(10133, null).read();
		}

	}

	private static void configureLoadBalancerIfEnabled(final Configuration configuration,
			final TCPConnector tcpConnector) {
		final boolean loadBalancerEnabled = configuration
				.getBooleanProperty(ConfigurationFactory.LOAD_BALANCER_ENABLED);

		if (loadBalancerEnabled) {
			new LoadBalancer(
					configuration.getStringProperty(ConfigurationFactory.LOAD_BALANCER_IP),
					configuration.getIntProperty(ConfigurationFactory.LOAD_BALANCER_PORT),
					configuration.getIntProperty(ConfigurationFactory.LOAD_BALANCER_WAIT_TIME),
					configuration
							.getStringProperty(ConfigurationFactory.LOAD_BALANCER_SCALING_GROUP),
					configuration, tcpConnector);
		} else {
			try {
				tcpConnector.connect();
			} catch (final IOException e) {
				e.printStackTrace();
			}
		}
	}
}
